@@ -1,6 +1,7 @@
 # Copyright (c) 2015 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
-from typing import List, Optional
+
+from typing import List, Optional, Tuple
 
 from UM.Signal import Signal
 from UM.Math.Vector import Vector
@@ -9,8 +10,6 @@ from UM.Scene.SceneNode import SceneNode
 
 from UM.Operations.GroupedOperation import GroupedOperation
 
-import copy
-
 
 ##    This class is responsible for keeping track of what objects are selected
 #     It uses signals to notify others of changes in the selection
@@ -29,10 +28,52 @@ class Selection:
     def remove(cls, object: SceneNode) -> None:
         if object in cls.__selection:
             cls.__selection.remove(object)
+            cls.unsetFace(object)
             object.transformationChanged.disconnect(cls._onTransformationChanged)
             cls._onTransformationChanged(object)
             cls.selectionChanged.emit()
 
+    @classmethod
+    def getFaceSelectMode(cls) -> bool:
+        return cls.__face_select_mode
+
+    @classmethod
+    def setFaceSelectMode(cls, select: bool) -> None:
+        cls.__face_select_mode = select
+        cls.selectedFaceChanged.emit()
+
+    @classmethod
+    def setFace(cls, object: SceneNode, face_id: int) -> None:
+        # Don't force-add the object, as the parent may be the 'actual' selected one.
+        cls.__selected_face = (object, face_id)
+        cls.selectedFaceChanged.emit()
+
+    @classmethod
+    def unsetFace(cls, object: Optional["SceneNode"] = None) -> None:
+        if not object or not cls.__selected_face or object is cls.__selected_face[0]:
+            cls.__selected_face = None
+            cls.selectedFaceChanged.emit()
+
+    @classmethod
+    def toggleFace(cls, object: SceneNode, face_id: int) -> None:
+        current_face = cls.__selected_face
+        if not current_face or object != current_face[0] or face_id != current_face[1]:
+            cls.setFace(object, face_id)
+        else:
+            cls.unsetFace(object)
+
+    @classmethod
+    def hoverFace(cls, object: SceneNode, face_id: int) -> None:
+        # Don't force-add the object, as the parent may be the 'actual' selected one.
+        cls.__hover_face = (object, face_id)
+        cls.hoverFaceChanged.emit()
+
+    @classmethod
+    def unhoverFace(cls, object: Optional["SceneNode"] = None) -> None:
+        if not object or not cls.__hover_face or object is cls.__hover_face[0]:
+            cls.__hover_face = None
+            cls.hoverFaceChanged.emit()
+
     @classmethod
     ##  Get number of selected objects
     def getCount(cls) -> int:
@@ -42,6 +83,14 @@ class Selection:
     def getAllSelectedObjects(cls) -> List[SceneNode]:
         return cls.__selection
 
+    @classmethod
+    def getSelectedFace(cls) -> Optional[Tuple[SceneNode, int]]:
+        return cls.__selected_face
+
+    @classmethod
+    def getHoverFace(cls) -> Optional[Tuple[SceneNode, int]]:
+        return cls.__hover_face
+
     @classmethod
     def getBoundingBox(cls) -> AxisAlignedBox:
         bounding_box = None  # don't start with an empty bounding box, because that includes (0,0,0)
@@ -75,6 +124,13 @@ class Selection:
         cls.__selection.clear()
         cls.selectionChanged.emit()
 
+    @classmethod
+    def clearFace(cls):
+        cls.__selected_face = None
+        cls.__hover_face = None
+        cls.selectedFaceChanged.emit()
+        cls.hoverFaceChanged.emit()
+
     @classmethod
     ##  Check if anything is selected at all.
     def hasSelection(cls) -> bool:
@@ -84,6 +140,10 @@ class Selection:
 
     selectionCenterChanged = Signal()
 
+    selectedFaceChanged = Signal()
+
+    hoverFaceChanged = Signal()
+
     @classmethod
     def getSelectionCenter(cls) -> Vector:
         return cls.__selection_center
@@ -129,3 +189,6 @@ class Selection:
 
     __selection = []    # type: List[SceneNode]
     __selection_center = Vector(0, 0, 0)
+    __selected_face = None    # type: Optional[Tuple[SceneNode, int]]
+    __hover_face = None    # type: Optional[Tuple[SceneNode, int]]
+    __face_select_mode = False
