@@ -19,6 +19,10 @@ from UM.Operations.GroupedOperation import GroupedOperation
 from UM.Operations.SetTransformOperation import SetTransformOperation
 from UM.Operations.LayFlatOperation import LayFlatOperation
 
+from UM.Version import Version
+
+from UM.View.GL.OpenGL import OpenGL
+
 from . import RotateToolHandle
 
 import math
@@ -48,9 +52,11 @@ class RotateTool(Tool):
         self._iterations = 0
         self._total_iterations = 0
         self._rotating = False
-        self.setExposedProperties("ToolHint", "RotationSnap", "RotationSnapAngle")
+        self.setExposedProperties("ToolHint", "RotationSnap", "RotationSnapAngle", "SelectFaceSupported")
         self._saved_node_positions = []
 
+        Selection.selectedFaceChanged.connect(self._onSelectedFaceChanged)
+
     ##  Handle mouse and keyboard events
     #
     #   \param event type(Event)
@@ -179,12 +185,53 @@ class RotateTool(Tool):
                     self.operationStopped.emit(self)
                 return True
 
+    def _onSelectedFaceChanged(self):
+        self._handle.setEnabled(not Selection.getFaceSelectMode())
+
+        selected_face = Selection.getSelectedFace()
+        if not Selection.getSelectedFace() or not (Selection.hasSelection() and Selection.getFaceSelectMode()):
+            return
+
+        original_node, face_id = selected_face
+        meshdata = original_node.getMeshDataTransformed()
+        if not meshdata or face_id < 0:
+            return
+
+        face_mid, face_normal = meshdata.getFacePlane(face_id)
+        object_mid = original_node.getBoundingBox().center
+        rotation_point_vector = Vector(object_mid.x, object_mid.y, face_mid[2])
+        face_normal_vector = Vector(face_normal[0], face_normal[1], face_normal[2])
+        rotation_quaternion = Quaternion.rotationTo(face_normal_vector.normalized(), Vector(0.0, -1.0, 0.0))
+
+        operation = GroupedOperation()
+        current_node = None  # type: Optional[SceneNode]
+        for node in Selection.getAllSelectedObjects():
+            current_node = node
+            parent_node = current_node.getParent()
+            while parent_node and parent_node.callDecoration("isGroup"):
+                current_node = parent_node
+                parent_node = current_node.getParent()
+        if current_node is None:
+            return
+
+        rotate_operation = RotateOperation(current_node, rotation_quaternion, rotation_point_vector)
+        operation.addOperation(rotate_operation)
+        operation.push()
+
+        # NOTE: We might want to consider unchecking the select-face button afterthe operation is done.
+
     ##  Return a formatted angle of the current rotate operation
     #
     #   \return type(String) fully formatted string showing the angle by which the mesh(es) are rotated
     def getToolHint(self):
         return "%d°" % round(math.degrees(self._angle)) if self._angle else None
 
+    ##  Get whether the select face feature is supported.
+    #   \return True if it is supported, or False otherwise.
+    def getSelectFaceSupported(self) -> bool:
+        # Use a dummy postfix, since an equal version with a postfix is considered smaller normally.
+        return Version(OpenGL.getInstance().getOpenGLVersion()) >= Version("4.1 dummy-postfix")
+
     ##  Get the state of the "snap rotation to N-degree increments" option
     #
     #   \return type(Boolean)
@@ -233,7 +280,6 @@ class RotateTool(Tool):
         self._total_iterations = 0
         for selected_object in self._getSelectedObjectsWithoutSelectedAncestors():
             self._layObjectFlat(selected_object)
-
         self._progress_message.show()
 
         operations = Selection.applyOperation(LayFlatOperation)
@@ -247,7 +293,7 @@ class RotateTool(Tool):
     ##  Lays the given object flat. The given object can be a group or not.
     def _layObjectFlat(self, selected_object):
         if not selected_object.callDecoration("isGroup"):
-            self._total_iterations += len(selected_object.getMeshDataTransformed().getVertices()) * 2
+            self._total_iterations += selected_object.getMeshData().getVertexCount() * 2
         else:
             for child in selected_object.getChildren():
                 self._layObjectFlat(child)
@@ -259,7 +305,7 @@ class RotateTool(Tool):
     def _layFlatProgress(self, iterations: int):
         self._iterations += iterations
         if self._progress_message:
-            self._progress_message.setProgress(100 * self._iterations / self._total_iterations)
+            self._progress_message.setProgress(min(100 * (self._iterations / self._total_iterations), 100))
 
     ##  Called when the LayFlatJob is done running all of its LayFlatOperations
     #
