@@ -7,11 +7,14 @@ from UM.Version import Version
 from UM.Logger import Logger
 from UM.Job import Job
 
+import ssl
 import urllib.request
 import platform
 import json
 import codecs
 
+import certifi
+
 from UM.i18n import i18nCatalog
 i18n_catalog = i18nCatalog("uranium")
 
@@ -36,8 +39,11 @@ class UpdateCheckerJob(Job):
         Logger.log("i", "Checking for new version of %s" % application_name)
         try:
             headers = {"User-Agent": "%s - %s" % (application_name, Application.getInstance().getVersion())}
+            # CURA-6698 Create an SSL context and use certifi CA certificates for verification.
+            context = ssl.SSLContext(protocol = ssl.PROTOCOL_TLSv1_2)
+            context.load_verify_locations(cafile = certifi.where())
             request = urllib.request.Request(self._url, headers = headers)
-            latest_version_file = urllib.request.urlopen(request)
+            latest_version_file = urllib.request.urlopen(request, context = context)
         except Exception as e:
             Logger.log("w", "Failed to check for new version: %s" % e)
             if not self.silent:
